"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MultilineTextDrawer = void 0;
const engine_1 = require("@tsparticles/engine");
class MultilineTextDrawer {
    draw(context, particle, radius, opacity) {
        var _a, _b, _c;
        const character = particle.shapeData;
        if (character === undefined) {
            return;
        }
        const textData = character.value;
        if (textData === undefined) {
            return;
        }
        const textParticle = particle;
        if (textParticle.text === undefined) {
            textParticle.text = (0, engine_1.itemFromSingleOrMultiple)(textData, particle.randomIndexData);
        }
        const text = textParticle.text, style = (_a = character.style) !== null && _a !== void 0 ? _a : "", weight = (_b = character.weight) !== null && _b !== void 0 ? _b : "400", size = Math.round(radius) * 2, font = (_c = character.font) !== null && _c !== void 0 ? _c : "Verdana", fill = particle.fill;
        context.font = `${style} ${weight} ${size}px "${font}"`;
        const lines = text === null || text === void 0 ? void 0 : text.split("\n");
        if (!lines) {
            return;
        }
        context.globalAlpha = opacity;
        for (let i = 0; i < lines.length; i++) {
            this._drawLine(context, lines[i], radius, opacity, i, fill);
        }
        context.globalAlpha = 1;
    }
    init(container) {
        return __awaiter(this, void 0, void 0, function* () {
            const options = container.options, shapeType = "multiline-text";
            if ((0, engine_1.isInArray)(shapeType, options.particles.shape.type)) {
                const shapeOptions = options.particles.shape.options[shapeType], promises = [];
                (0, engine_1.executeOnSingleOrMultiple)(shapeOptions, (shape) => {
                    promises.push((0, engine_1.loadFont)(shape.font, shape.weight));
                });
                yield Promise.all(promises);
            }
        });
    }
    _drawLine(context, line, radius, opacity, index, fill) {
        const offsetX = (line.length * radius) / 2, pos = {
            x: -offsetX,
            y: radius / 2,
        };
        if (fill) {
            context.fillText(line, pos.x, pos.y + radius * 2 * index);
        }
        else {
            context.strokeText(line, pos.x, pos.y + radius * 2 * index);
        }
    }
}
exports.MultilineTextDrawer = MultilineTextDrawer;
