import { executeOnSingleOrMultiple, isInArray, itemFromSingleOrMultiple, loadFont } from "@tsparticles/engine";
export class MultilineTextDrawer {
    draw(context, particle, radius, opacity) {
        var _a, _b, _c;
        const character = particle.shapeData;
        if (character === undefined) {
            return;
        }
        const textData = character.value;
        if (textData === undefined) {
            return;
        }
        const textParticle = particle;
        if (textParticle.text === undefined) {
            textParticle.text = itemFromSingleOrMultiple(textData, particle.randomIndexData);
        }
        const text = textParticle.text, style = (_a = character.style) !== null && _a !== void 0 ? _a : "", weight = (_b = character.weight) !== null && _b !== void 0 ? _b : "400", size = Math.round(radius) * 2, font = (_c = character.font) !== null && _c !== void 0 ? _c : "Verdana", fill = particle.fill;
        context.font = `${style} ${weight} ${size}px "${font}"`;
        const lines = text === null || text === void 0 ? void 0 : text.split("\n");
        if (!lines) {
            return;
        }
        context.globalAlpha = opacity;
        for (let i = 0; i < lines.length; i++) {
            this._drawLine(context, lines[i], radius, opacity, i, fill);
        }
        context.globalAlpha = 1;
    }
    async init(container) {
        const options = container.options, shapeType = "multiline-text";
        if (isInArray(shapeType, options.particles.shape.type)) {
            const shapeOptions = options.particles.shape.options[shapeType], promises = [];
            executeOnSingleOrMultiple(shapeOptions, (shape) => {
                promises.push(loadFont(shape.font, shape.weight));
            });
            await Promise.all(promises);
        }
    }
    _drawLine(context, line, radius, opacity, index, fill) {
        const offsetX = (line.length * radius) / 2, pos = {
            x: -offsetX,
            y: radius / 2,
        };
        if (fill) {
            context.fillText(line, pos.x, pos.y + radius * 2 * index);
        }
        else {
            context.strokeText(line, pos.x, pos.y + radius * 2 * index);
        }
    }
}
